#!/usr/bin/env python3

import json
import logging
import os
import sys


class GitlabDASTReport:
    def __init__(self, n0s1_data=None):
        tool_name = n0s1_data.get("tool", {}).get("name", "")
        tool_version = n0s1_data.get("tool", {}).get("version", "")
        prvd = n0s1_data.get("tool", {}).get("author", "")
        self.vulns: list[dict] = []
        self.report = {
            "version": "14.1.2",
            "vulnerabilities": self.vulns,
            "scan": {
                "start_time": "2000-01-01T00:00:00",
                "end_time": "2000-01-01T00:30:00",
                "status": "success",
                "type": "dast",
                "scanner": {
                    "id": tool_name,
                    "name": f"{tool_name} secret scanner by {prvd} - version: [{tool_version}]",
                    "url": "https://spark1.us/n0s1",
                    "version": tool_version,
                    "vendor": {
                        "name": prvd,
                    },
                },
            },
        }
        if n0s1_data:
            self.add_vulns(n0s1_data)

    def add_vulns(self, n0s1_data: dict):
        findings = n0s1_data.get("findings", [])
        for key in findings:
            d = findings[key]
            try:
                finding_instance_id = d.get("id", "")
                url = d.get("url", "")
                secret = d.get("secret", "")
                platform = d.get("details", {}).get("platform", "PM software")
                field = d.get("details", {}).get("ticket_field", "ticket")
                match = d.get("details", {}).get("matched_regex_config", {})
                match_id = match.get("id", "")
                match_description = match.get("description", "")

                finding_message = f"Potential Secret Leak on {platform} {field}"
                message = finding_message
                message += f"\nDetails:\nSensitive data type: [{match_id}] - Description: [{match_description}]\nPlatform: [{platform}] - Field: [ticket {field}]\nSource: {url}"
                message += f"\n000000000000000 Sensitive data found (redacted) 000000000000000\n{secret}\n000000000000000 Sensitive data found (redacted) 000000000000000"
                message += f"\nPlease verify the {platform} ticket and conduct a thorough search for any sensitive data. If a data leak is confirmed, proceed to rotate the data and eliminate any sensitive information from the ticket."
                finding_description = message.replace("<REDACTED>", "xxxxxxxxxxxx")

                severity = "Info"
                finding_location_url = f"{platform} ticket {field}: {url}"
                identifiers_name = match_description
                identifiers_value = match_id

                self.vulns.append(
                    {
                        "id": finding_instance_id,
                        "category": "dast",
                        "message": finding_message,
                        "description": finding_description,
                        "cve": "",
                        "severity": severity,
                        "scanner": {
                            "id": "n0s1",
                            "name": "n0s1",
                        },
                        "location": {
                            "file": finding_location_url,
                            "start_line": 0
                        },
                        "identifiers": [
                            {
                                "type": "n0s1",
                                "name": identifiers_name,
                                "url": "https://spark1.us/n0s1",
                                "value": identifiers_value
                            }
                        ]
                    }
                )
            except KeyError:
                logging.info("Warning! Unexpected JSON format")
                pass
            except Exception as e:
                logging.info(str(e))

    def write_gitlab_report(self, file="gl-dast-report.json") -> None:
        with open(file, "w") as f:
            f.write(json.dumps(self.report))


def n0s1_report_to_gitlab_report(n0s1_report):
    return GitlabDASTReport(n0s1_report)


def n0s1_report_file_to_gitlab_report_file(input_report_path, output_report_path):
    if os.path.exists(input_report_path):
        with open(input_report_path) as f:
            data = json.load(f)
            logging.info(f"Parsing report file [{output_report_path}]...")
            gitlab_report = GitlabDASTReport(data)
            if len(gitlab_report.vulns) <= 0:
                logging.info(f"No leaks found on file: [{output_report_path}].")
            gitlab_report.write_gitlab_report(output_report_path)
    else:
        logging.info(f"Skipping report file: [{output_report_path}]")


if __name__ == "__main__":
    logging.basicConfig(format="[%(levelname)s] %(message)s", level=logging.INFO)

    INPUT_REPORT_PATH = "n0s1_report.json"
    OUTPUT_REPORT_PATH = "gl-dast-report.json"

    if len(sys.argv) > 1:
        INPUT_REPORT_PATH = sys.argv[1]
    if len(sys.argv) > 2:
        OUTPUT_REPORT_PATH = sys.argv[2]

    n0s1_report_file_to_gitlab_report_file(INPUT_REPORT_PATH, OUTPUT_REPORT_PATH)


import logging


class JiraControler():
    def __init__(self):
        self._client = None

    def set_config(self, config):
        from jira import JIRA
        SERVER = config.get("server", "")
        EMAIL = config.get("email", "")
        TOKEN = config.get("token", "")
        if EMAIL and len(EMAIL) > 0:
            self._client = JIRA(SERVER, basic_auth=(EMAIL, TOKEN))
        else:
            self._client = JIRA(SERVER, token_auth=TOKEN)
        return self.is_connected()

    def get_name(self):
        return "Jira"

    def is_connected(self):
        if self._client:
            user = self._client.myself()
            if user:
                logging.info(f"Logged to {self.get_name()} as {user}")
            else:
                logging.error(f"Unable to connect to {self.get_name()} instance. Check your credentials.")
                return False

            projects = self._client.projects()

            if projects:
                project_found = False
                issue_found = False
                for key in projects:
                    project_found = True
                    ql = f"project = {key}"
                    for issue in self._client.search_issues(ql):
                        if issue:
                            issue_found = True
                            return True
                if project_found:
                    if issue_found:
                        return True
                    else:
                        logging.error(f"Unable to list {self.get_name()} issues. Check your permissions.")
                else:
                    logging.error(f"Unable to list {self.get_name()} projects. Check your permissions.")
            else:
                logging.error(f"Unable to connect to {self.get_name()} instance. Check your credentials.")
        return False

    def get_data(self, include_coments=False):
        if not self._client:
            return None, None, None, None, None
        for key in self._client.projects():
            ql = f"project = {key}"
            for issue in self._client.search_issues(ql):
                url = issue.self.split('/rest/api')[0] + "/browse/" + issue.key;
                title = issue.fields.summary
                description = issue.fields.description
                comments = []
                if include_coments:
                    issue_comments = self._client.comments(issue.id)
                    for c in issue_comments:
                        comments.append(c.body)
                yield title, description, comments, url, issue.key

    def post_comment(self, issue, comment):
        if not self._client:
            return False
        comment = comment.replace("#", "0")
        comment_status = self._client.add_comment(issue, body=comment)
        status = comment_status.id
        if status and len(status) > 0 and int(status) > 0:
            return True
        return False



#!/usr/bin/env python3

import json
import logging
import os
import sys


def _n0s1_rule_2_sarif_rule(n0s1_data):
    sarif_rules = []
    n0s1_rules = n0s1_data.get("regex_config", {}).get("rules", [])
    for r in n0s1_rules:
        id = r.get("id", "")
        description = r.get("description", "")
        sarif_r = {
            "id": id,
            "name": description,
            "shortDescription": {
                "text": description
            },
            "fullDescription": {
                "text": f"{description}"
            },
            "defaultConfiguration": {
                "level": "note"
            },
            "helpUri": "https://spark1.us/n0s1"
        }
        sarif_rules.append(sarif_r)
    return sarif_rules


class SarifReport:
    def __init__(self, n0s1_data) -> None:
        tool_name = n0s1_data.get("tool", {}).get("name", "")
        tool_version = n0s1_data.get("tool", {}).get("version", "")
        prvd = n0s1_data.get("tool", {}).get("author", "")
        self.results: list[dict] = []
        self.rules: list[dict] = []
        self.seen_rules: set[str] = set()
        self.rules = _n0s1_rule_2_sarif_rule(n0s1_data)

        for rule in self.rules:
            self.seen_rules.add(rule["id"])

        self.report = {
            "version": "2.1.0",
            "$schema": "http://json.schemastore.org/sarif-2.1.0-rtm.4",
            "runs": [
                {
                    "tool": {"driver": {"name": tool_name, "rules": self.rules, "version": tool_version,
                                        "fullName": f"{tool_name} secret scanner by {prvd} - version: [{tool_version}]",
                                        "informationUri": "https://github.com/spark1security/n0s1"}
                             },
                    "results": self.results,
                }
            ],
        }
        if n0s1_data:
            self.add_vulns(n0s1_data)

    def get_rule_index(self, rule_id):
        index = 0
        for r in self.rules:
            index += 1
            if rule_id.lower() == r.get("id", "").lower():
                return index
        return -1

    def add_vulns(self, n0s1_data: dict):
        findings = n0s1_data.get("findings", [])
        for key in findings:
            d = findings[key]
            try:
                url = d.get("url", "")
                secret = d.get("secret", "")
                platform = d.get("details", {}).get("platform", "PM software")
                field = d.get("details", {}).get("ticket_field", "ticket")
                level = "note"
                match = d.get("details", {}).get("matched_regex_config", {})
                match_id = match.get("id", "")
                ruleId = match_id
                match_description = match.get("description", "")
                ruleIndex = self.get_rule_index(ruleId)

                if ruleId not in self.seen_rules:
                    self.rules.append(
                        {
                            "id": ruleId,
                            "name": f"Secret Leak - {ruleId}",
                            "shortDescription": {"text": ruleId},
                            "fullDescription": {"text": ruleId},
                            "defaultConfiguration": {"level": "note"},
                            "helpUri": "https://spark1.us/n0s1"
                        }
                    )

                message = f"Potential Secret Leak on [{platform}]({url})."
                message += f"\nDetails:\nSensitive data type: [{match_id}] - Description: [{match_description}]\nPlatform: [{platform}] - Field: [ticket {field}]\nSource: {url}"
                message += f"\n000000000000000 Sensitive data found (redacted) 000000000000000\n{secret}\n000000000000000 Sensitive data found (redacted) 000000000000000"
                message += f"\nPlease verify the [{platform} ticket]({url}) and conduct a thorough search for any sensitive data. If a data leak is confirmed, proceed to rotate the data and eliminate any sensitive information from the ticket."
                message = message.replace("<REDACTED>", "xxxxxxxxxxxx")

                self.results.append(
                    {
                        "ruleId": ruleId,
                        "ruleIndex": ruleIndex,
                        "message": {"text": message},
                        "level": level,
                        "locations": [
                            {
                                "physicalLocation": {
                                    "artifactLocation": {"uri": "None", "uriBaseId": "ROOTPATH"},
                                    "region": {
                                        "startLine": 1,
                                        "startColumn": 1,
                                        "endLine": 1,
                                        "endColumn": 1
                                    }
                                },
                                "message": {
                                    "text": url
                                }
                            }
                        ]
                    }
                )
            except KeyError:
                logging.info("Warning! Unexpected JSON format")
                pass
            except Exception as e:
                logging.info(str(e))

    def write_report(self, file="report.sarif") -> None:
        existing_sarif_data = None
        if os.path.exists(file):
            with open(file) as f:
                existing_sarif_data = json.load(f)

        result = None
        if existing_sarif_data and "runs" in existing_sarif_data and "runs" in self.report:
            result = existing_sarif_data
            result["runs"] += self.report["runs"]
        else:
            result = self.report

        with open(file, "w") as f:
            f.write(json.dumps(result))
            logging.info(f"Secret scanner report saved to: [{file}].")


def n0s1_report_to_sarif_report(n0s1_report):
    return SarifReport(n0s1_report)


def n0s1_report_file_to_sarif_report_file(input_report_path, output_report_path):
    if os.path.exists(input_report_path):
        with open(input_report_path) as f:
            data = json.load(f)
            logging.info(f"Parsing report file [{input_report_path}]...")
            sarif_report = n0s1_report_to_sarif_report(data)
            if len(sarif_report.results) <= 0:
                logging.info(f"No leaks found on file: [{input_report_path}].")
            sarif_report.write_report(output_report_path)
    else:
        logging.info(f"Skipping report file: [{input_report_path}]")


if __name__ == "__main__":
    logging.basicConfig(format="[%(levelname)s] %(message)s", level=logging.INFO)

    INPUT_REPORT_PATH = "n0s1_report.json"
    OUTPUT_REPORT_PATH = "n0s1_report.sarif"

    if len(sys.argv) > 1:
        INPUT_REPORT_PATH = sys.argv[1]
    if len(sys.argv) > 2:
        OUTPUT_REPORT_PATH = sys.argv[2]

    n0s1_report_file_to_sarif_report_file(INPUT_REPORT_PATH, OUTPUT_REPORT_PATH)



class PlatformHollow:
    def __init__(self):
        self.client = None

    def set_config(self, config):
        return self.is_connected()

    def get_name(self):
        return "Hollow"

    def is_connected(self):
        return False

    def get_data(self, include_coments=False):
        return None, None, None, None, None

    def post_comment(self, issue, comment):
        return self.is_connected()


class PlatformFactory:
    def __init__(self):
        self._creators = {}

    def register_platform(self, platform, creator):
        self._creators[platform] = creator

    def get_platform(self, platform):
        creator = self._creators.get(platform)
        if creator:
            return creator()
        return PlatformHollow()


global factory
factory = PlatformFactory()

try:
    from . import jira_controller as jira_controller
    from . import confluence_controller as confluence_controller
    from . import linear_controller as linear_controller
except:
    import n0s1.controllers.jira_controller as jira_controller
    import n0s1.controllers.confluence_controller as confluence_controller
    import n0s1.controllers.linear_controller as linear_controller

factory.register_platform("", jira_controller.JiraControler)
factory.register_platform("jira", jira_controller.JiraControler)
factory.register_platform("jira_scan", jira_controller.JiraControler)
factory.register_platform("confluence", confluence_controller.ConfluenceControler)
factory.register_platform("confluence_scan", confluence_controller.ConfluenceControler)
factory.register_platform("linear", linear_controller.LinearControler)
factory.register_platform("linear_scan", linear_controller.LinearControler)



import argparse
import hashlib
import logging
import json
import os
import pathlib
import re
import sys
import toml
import yaml
from datetime import datetime, timezone

try:
    import controllers.platform_controller as platform_controller
except:
    import n0s1.controllers.platform_controller as platform_controller

try:
    import reporting.report_sarif as report_sarif
except:
    import n0s1.reporting.report_sarif as report_sarif

try:
    import reporting.report_gitlab as report_gitlab
except:
    import n0s1.reporting.report_gitlab as report_gitlab


global report_json, report_file, cfg, DEBUG


def init_argparse() -> argparse.ArgumentParser:
    """Adds arguements that can be called from the command line

    Returns:
        argparse.ArgumentParser: A parser that contains the command line options
    """
    install_path = os.path.dirname(os.path.abspath(__file__))
    parser = argparse.ArgumentParser(
        prog="n0s1",
        description="""Secret scanner for Project Management platforms such as Jira, Confluence and Linear.
        """,
    )

    # Create parent subparser. Note `add_help=False` and creation via `argparse.`
    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parser.add_argument(
        "--regex-file",
        dest="regex_file",
        nargs="?",
        default=f"{install_path}/config/regex.toml",
        type=str,
        help="Custom .toml with a list of regexes to be matched."
    )
    parent_parser.add_argument(
        "--config-file",
        dest="config_file",
        nargs="?",
        default=f"{install_path}/config/config.yaml",
        type=str,
        help="Configuration file (YAML format) to be used."
    )
    parent_parser.add_argument(
        "--report-file",
        dest="report_file",
        nargs="?",
        default="n0s1_report.json",
        type=str,
        help="Output report file for the leaked secrets."
    )
    parent_parser.add_argument(
        "--report-format",
        dest="report_format",
        nargs="?",
        default="n0s1",
        type=str,
        help="Output report format. Supported formats: n0s1, SARIF, gitlab."
    )
    parent_parser.add_argument(
        "--post-comment",
        dest="post_comment",
        action="store_true",
        help="By default, scans only flag leaked secrets; this adds a warning comment to every ticket with a potential secret leak",
    )
    parent_parser.add_argument(
        "--skip-comment",
        dest="skip_comment",
        action="store_true",
        help="By default, scans check the ticket title, description and comments; this flag disables ticket comment scanning",
    )
    parent_parser.add_argument(
        "--show-matched-secret-on-logs",
        dest="show_matched_secret_on_logs",
        action="store_true",
        help="By default, only a sanitized version of the leak is shown on logs. This flag makes the actual leaked secret to be displayed on logs. Be extra careful when enabling this flag because you might make the leak worst by sending sensitive info to logs.",
    )
    parent_parser.add_argument(
        "--debug",
        dest="debug",
        action="store_true",
        help="Debug mode. Warning it may further expose sensitive data.",
    )
    parent_parser.add_argument(
        "--secret-manager",
        dest="secret_manager",
        nargs="?",
        default="a secret manager tool",
        type=str,
        help="Secret manager tool name to be suggested when leaks are found."
    )
    parent_parser.add_argument(
        "--contact-help",
        dest="contact_help",
        nargs="?",
        default="contact@spark1.us",
        type=str,
        help="Contact information for assistance when leaks are detected."
    )
    parent_parser.add_argument(
        "--label",
        dest="label",
        nargs="?",
        default="n0s1bot_auto_comment_e869dd5fa15ca0749a350aac758c7f56f56ad9be1",
        type=str,
        help="Unique identifier to be added to the comments so that the n0s1 bot can recognize if the leak has been previously flagged."
    )
    subparsers = parser.add_subparsers(
        help="Subcommands", dest="command", metavar="COMMAND"
    )

    linear_scan_parser = subparsers.add_parser(
        "linear_scan", help="Scan Linear tickets", parents=[parent_parser]
    )
    linear_scan_parser.add_argument(
        "--api-key",
        dest="api_key",
        nargs="?",
        type=str,
        help="Linear API key. Ref: https://developers.linear.app/docs/graphql/working-with-the-graphql-api#personal-api-keys"
    )

    jira_scan_parser = subparsers.add_parser(
        "jira_scan", help="Scan Jira tickets", parents=[parent_parser]
    )
    jira_scan_parser.add_argument(
        "--server",
        dest="server",
        nargs="?",
        type=str,
        help="Jira server uri."
    )
    jira_scan_parser.add_argument(
        "--email",
        dest="email",
        nargs="?",
        type=str,
        help="Jira user email."
    )
    jira_scan_parser.add_argument(
        "--api-key",
        dest="api_key",
        nargs="?",
        type=str,
        help="Jira API key."
    )

    confluence_scan_parser = subparsers.add_parser(
        "confluence_scan", help="Scan Confluence pages", parents=[parent_parser]
    )
    confluence_scan_parser.add_argument(
        "--server",
        dest="server",
        nargs="?",
        type=str,
        help="Confluence base URL (e.g. https://yourcompany.atlassian.net)."
    )
    confluence_scan_parser.add_argument(
        "--email",
        dest="email",
        nargs="?",
        type=str,
        help="Confluence user email."
    )
    confluence_scan_parser.add_argument(
        "--api-key",
        dest="api_key",
        nargs="?",
        type=str,
        help="Confluence API key."
    )
    return parser


def _sanitize_text(text, begin, end):
    text_len = len(text)
    s_begin = max(begin-20, 0)
    s_end = min(end+20, text_len)
    sanitized_text = text[s_begin:begin] + "<REDACTED>" + text[end:s_end]
    snippet_text = text[s_begin:s_end]
    return sanitized_text, snippet_text


def _sha1_hash(to_hash):
    try:
        message_digest = hashlib.sha1()
        string_m = str(to_hash)
        byte_m = bytes(string_m, encoding='utf')
        message_digest.update(byte_m)
        return message_digest.hexdigest()
    except TypeError:
        raise "Unable to generate SHA-1 hash for input string"


def _save_report(scan_text_result):
    global report_json, report_file

    report_format = scan_text_result.get("scan_arguments", {}).get("report_format", "n0s1")

    try:
        if report_format.lower().find("sarif".lower()) != -1:
            github_report = report_sarif.n0s1_report_to_sarif_report(report_json)
            github_report.write_report(report_file)
            return True
        elif report_format.lower().find("gitlab".lower()) != -1:
            gitlab_report = report_gitlab.n0s1_report_to_gitlab_report(report_json)
            gitlab_report.write_report(report_file)
            return True
        else:
            with open(report_file, "w") as f:
                json.dump(report_json, f)
                return True
    except Exception as e:
        logging.error(e)

    return False


def match_regex(regex_config, text):
    for c in regex_config["rules"]:
        regex_str = c["regex"]
        m = re.search(regex_str, text)
        if m:
            begin = m.regs[0][0]
            end = m.regs[0][1]
            matched_text = text[begin:end]
            sanitized_text, snippet_text = _sanitize_text(text, begin, end)
            return c, matched_text, sanitized_text, snippet_text
    return None, None, None, None


def report_leaked_secret(scan_text_result, controller):
    global report_json, cfg
    snippet_text = scan_text_result.get("snippet_text", "")
    sanitized_secret = scan_text_result.get("sanitized_secret", "")
    matched = scan_text_result.get("matched_regex_config", {})
    regex_id = matched.get("id", "")
    regex_description = matched.get("description", "")
    regex = matched.get("regex", "")
    platform = scan_text_result.get("ticket_data", {}).get("platform", "")
    field = scan_text_result.get("ticket_data", {}).get("field", "")
    url = scan_text_result.get("ticket_data", {}).get("url", "")
    issue_id = scan_text_result.get("ticket_data", {}).get("issue_id", "")
    post_comment = scan_text_result.get("scan_arguments", {}).get("post_comment", False)
    show_matched_secret_on_logs = scan_text_result.get("scan_arguments", {}).get("show_matched_secret_on_logs", False)

    finding_info = "Platform:[{platform}] Field:[ticket {field}] ID:[{regex_config_id}] Description:[{regex_config_description}] Regex: {regex}\n############## Sanitized Secret Leak ##############\n {leak}\n############## Sanitized Secret Leak ##############"
    finding_info = finding_info.format(regex_config_id=regex_id, regex_config_description=regex_description,
                                       regex=regex, platform=platform, field=field, leak=sanitized_secret)

    logging.info("\nPotential secret leak regex match!")
    logging.info(finding_info)
    if show_matched_secret_on_logs:
        logging.info(f"\n##################### Secret  #####################\n{snippet_text}\n##################### Secret  #####################")
    logging.info(f"\nLeak source: {url}")
    logging.info("\n\n")
    finding_id = url + "_" + sanitized_secret
    finding_id = _sha1_hash(finding_id)
    new_finding = {"id": finding_id, "url": url, "secret": sanitized_secret,
                   "details": {"matched_regex_config": scan_text_result["matched_regex_config"], "platform": platform, "ticket_field": field}}
    if finding_id not in report_json["findings"]:
        report_json["findings"][finding_id] = new_finding
        _save_report(scan_text_result)
    if post_comment:
        comment_template = cfg.get("comment_params", {}).get("message_template", "")
        bot_name = cfg.get("comment_params", {}).get("bot_name", "bot")
        secret_manager = scan_text_result.get("scan_arguments", {}).get("secret_manager", "")
        contact_help = scan_text_result.get("scan_arguments", {}).get("contact_help", "")
        label = scan_text_result.get("scan_arguments", {}).get("label", "")
        format_variables = ["finding_info", "bot_name", "secret_manager", "contact_help", "label"]
        for variable in format_variables:
            if comment_template.find(variable) == -1:
                comment_template += f"\n{variable}: {{{variable}}}"
        comment = comment_template.format(finding_info=finding_info, bot_name=bot_name, secret_manager=secret_manager,
                                          contact_help=contact_help, label=label)
        return controller.post_comment(issue_id, comment)
    return True


def scan_text(regex_config, text):
    global DEBUG
    match = False
    scan_text_result = {}
    try:
        matched_regex_config, secret, sanitized_secret, snippet_text = match_regex(regex_config, str(text))
        scan_text_result = {"matched_regex_config": matched_regex_config, "secret": secret,
                            "sanitized_secret": sanitized_secret, "snippet_text": snippet_text}
        if matched_regex_config:
            match = True
    except Exception as e:
        if DEBUG:
            logging.warning(e)
        pass

    return match, scan_text_result


def scan(regex_config, controller, scan_arguments):
    if not regex_config or not controller:
        return
    scan_comment = scan_arguments.get("scan_comment", False)
    post_comment = scan_arguments.get("post_comment", False)
    for title, description, comments, url, issue_id in controller.get_data(scan_comment):
        logging.info(f"Scanning [{issue_id}]: {url}")
        ticket_data = {"title": title, "description": description, "comments": comments, "url": url, "issue_id": issue_id}
        label = cfg.get("comment_params", {}).get("label", "")
        post_comment_for_this_issue = post_comment
        if post_comment_for_this_issue:
            for comment in comments:
                if comment.lower().find(label.lower()) != -1:
                    # Comment with leak warning has been already posted. Skip
                    # posting a new comment again
                    post_comment_for_this_issue = False
                    break
        scan_arguments["post_comment"] = post_comment_for_this_issue

        secret_found, scan_text_result = scan_text(regex_config, title)
        scan_text_result["ticket_data"] = ticket_data
        scan_text_result["ticket_data"]["field"] = "title"
        scan_text_result["ticket_data"]["platform"] = controller.get_name()
        scan_text_result["scan_arguments"] = scan_arguments
        if secret_found:
            report_leaked_secret(scan_text_result, controller)

        secret_found, scan_text_result = scan_text(regex_config, description)
        scan_text_result["ticket_data"] = ticket_data
        scan_text_result["ticket_data"]["field"] = "description"
        scan_text_result["ticket_data"]["platform"] = controller.get_name()
        scan_text_result["scan_arguments"] = scan_arguments
        if secret_found:
            report_leaked_secret(scan_text_result, controller)

        for comment in comments:
            secret_found, scan_text_result = scan_text(regex_config, comment)
            scan_text_result["ticket_data"] = ticket_data
            scan_text_result["ticket_data"]["field"] = "comment"
            scan_text_result["ticket_data"]["platform"] = controller.get_name()
            scan_text_result["scan_arguments"] = scan_arguments
            if secret_found:
                report_leaked_secret(scan_text_result, controller)


def main():
    global report_json, report_file, cfg, DEBUG

    logging.basicConfig(level=logging.INFO)
    parser = init_argparse()
    args = parser.parse_args()

    regex_config = None
    cfg = {}

    if not args.command:
        parser.print_help()
        return

    if os.path.exists(args.regex_file):
        with open(args.regex_file, "r") as f:
            regex_config = toml.load(f)
    else:
        logging.warning(f"Regex file [{args.regex_file}] not found!")

    if os.path.exists(args.config_file):
        with open(args.config_file, "r") as f:
            cfg = yaml.load(f, Loader=yaml.FullLoader)
    else:
        logging.warning(f"Config file [{args.config_file}] not found!")

    datetime_now_obj = datetime.now(timezone.utc)
    date_utc = datetime_now_obj.strftime("%Y-%m-%d %H:%M:%S")
    try:
        here = pathlib.Path(__file__).parent.resolve()
        init_file = pathlib.Path(here / "__init__.py")
        n0s1_version = re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]', init_file.read_text(), re.M).group(1)
    except:
        n0s1_version = "0.0.1"
    report_json = {"tool": {"name": "n0s1", "version": n0s1_version, "author": "Spark 1 Security"},
                   "scan_date": {"timestamp": datetime_now_obj.timestamp(),"date_utc": date_utc},
                   "regex_config": regex_config, "findings": {}}
    report_file = args.report_file

    command = args.command
    controller_factory = platform_controller.factory
    controller = controller_factory.get_platform(command)

    TOKEN = None
    SERVER = None
    EMAIL = None
    if command == "linear_scan":
        TOKEN = os.getenv("LINEAR_TOKEN")
        if args.api_key and len(args.api_key) > 0:
            TOKEN = args.api_key
        controler_config = {"token": TOKEN}

    elif command == "jira_scan":
        SERVER = os.getenv("JIRA_SERVER")
        EMAIL = os.getenv("JIRA_EMAIL")
        TOKEN = os.getenv("JIRA_TOKEN")
        if args.server and len(args.server) > 0:
            SERVER = args.server
        if args.email and len(args.email) > 0:
            EMAIL = args.email
        if args.api_key and len(args.api_key) > 0:
            TOKEN = args.api_key
        controler_config = {"server": SERVER, "email": EMAIL, "token": TOKEN}

    elif command == "confluence_scan":
        SERVER = os.getenv("CONFLUENCE_SERVER")
        if not SERVER:
            SERVER = os.getenv("JIRA_SERVER")
        EMAIL = os.getenv("CONFLUENCE_EMAIL")
        if not EMAIL:
            EMAIL = os.getenv("JIRA_EMAIL")
        TOKEN = os.getenv("CONFLUENCE_TOKEN")
        if not TOKEN:
            TOKEN = os.getenv("JIRA_TOKEN")
        if args.server and len(args.server) > 0:
            SERVER = args.server
        if args.email and len(args.email) > 0:
            EMAIL = args.email
        if args.api_key and len(args.api_key) > 0:
            TOKEN = args.api_key
        controler_config = {"server": SERVER, "email": EMAIL, "token": TOKEN}

    else:
        parser.print_help()
        return

    message = f"n0s1 secret scanner version [{n0s1_version}] - Scan date: {date_utc}"
    logging.info(message)
    DEBUG = args.debug
    if DEBUG:
        message = f"Args: {args}"
        logging.info(message)
        message = f"Controller settings: {SERVER} {EMAIL}"
        if args.show_matched_secret_on_logs:
            message += f" {TOKEN}"
        logging.info(message)

    if not controller.set_config(controler_config):
        sys.exit(-1)

    if args.post_comment:
        post_comment = args.post_comment
    else:
        post_comment = cfg.get("general_params", {}).get("post_comment", False)

    if args.skip_comment:
        skip_comment = args.skip_comment
    else:
        skip_comment = cfg.get("general_params", {}).get("skip_comment", False)
    scan_comment = not skip_comment

    if args.secret_manager:
        secret_manager = args.secret_manager
    else:
        secret_manager = cfg.get("comment_params", {}).get("secret_manager", False)

    if args.contact_help:
        contact_help = args.contact_help
    else:
        contact_help = cfg.get("comment_params", {}).get("contact_help", False)

    if args.label:
        label = args.label
    else:
        label = cfg.get("comment_params", {}).get("label", False)

    if args.show_matched_secret_on_logs:
        show_matched_secret_on_logs = args.show_matched_secret_on_logs
    else:
        show_matched_secret_on_logs = cfg.get("general_params", {}).get("show_matched_secret_on_logs", False)

    if args.report_format:
        report_format = args.report_format
    else:
        report_format = cfg.get("general_params", {}).get("report_format", False)

    scan_arguments = {"scan_comment": scan_comment, "post_comment": post_comment, "secret_manager": secret_manager,
                      "contact_help": contact_help, "label": label, "report_format": report_format, "debug": DEBUG,
                      "show_matched_secret_on_logs": show_matched_secret_on_logs}
    scan(regex_config, controller, scan_arguments)

    logging.info("Done!")


if __name__ == "__main__":
    main()



import logging

try:
    import clients.linear_graphql_client as linear_graphql_client
except:
    import n0s1.clients.linear_graphql_client as linear_graphql_client


class LinearControler():
    def __init__(self):
        self._client = None

    def set_config(self, config):
        TOKEN = config.get("token", "")
        headers = {
            "Content-Type": "application/json",
            "Authorization": TOKEN,
        }
        self._client = linear_graphql_client.LinearGraphQLClient(
            logging=logging,
            headers=headers,
        )
        return self.is_connected()

    def get_name(self):
        return "Linear"

    def is_connected(self):
        if self._client:
            user = self._client.get_curret_user()
            if user:
                logging.info(f"Logged to Linear as {user}")
            else:
                logging.error(f"Unable to connect to Linear instance. Check your credentials.")
                return False

            query = {"query": "{ issues { nodes { id } } }", "variables": {}}
            r = self._client.graphql_query(query)
            if r.status_code == 200:
                data = r.json()
                issues = data.get("data", {}).get("issues", {}).get("nodes", [])
                if len(issues) > 0:
                    return True
                else:
                    logging.error(f"Unable to list Linear issues. Check your permissions.")
            else:
                logging.error(f"Unable to connect to Linear instance. Check your credentials.")
        return False

    def get_data(self, include_coments=False):
        if not self._client:
            return None, None, None, None, None
        for linear_data in self._client.get_issues_and_comments(20):
            for edge in linear_data.get("data", {}).get("issues", {}).get("edges", []):
                item = edge.get("node", {})
                url = item.get("url", "")
                title = item.get("title", "")
                description = item.get("description", "")
                issue_key = item.get("identifier", "")
                comments = []
                if include_coments:
                    for node in item.get("comments", {}).get("nodes", []):
                        comment = node.get("body", "")
                        if len(comment) > 0:
                            comments.append(comment)
                yield title, description, comments, url, issue_key

    def post_comment(self, issue, comment):
        if not self._client:
            return False
        comment_status = self._client.add_comment(issue, comment)
        return comment_status.get("data", {}).get("commentCreate", {}).get("success", False)